using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    static class LeftRight
    {
       public const int left =1;
       public const int right =2;
       public const int both = 3;
    }

    static class TopMenuTabSelection
    {
        public const int loadPlugins = 1;
        public const int findPlugins = 2;
        public const int categories = 3;
        public const int settings = 4;
    }

    static class CatMenuTabSelection
    {
        public const int pluginCat = 1;
        public const int creatorCat = 2;
        public const int pluginTypes = 3;
        public const int pluginTags = 4;
    }

    static class SettingsMenuTabSelection
    {
        public const int pluginFilter = 1;
        public const int general = 2;
        public const int about = 3;
    }

    static class ConcealedFilter
    {
        public const int concealed = 1;
        public const int notConcealed = 2;
        public const int any = 3;
    }

    static class PreferredFilter
    {
        public const int preferred = 1;
        public const int notPreferred = 2;
        public const int any = 3;
    }

    static class PluginAtomTypes
    {
        public const int All = 0;
        public const int General = 1;

        public const int PersonFemale = 2;
        public const int PersonMale = 3;
        public const int PersonAny = 4;

        public const int CUA = 5;
        public const int Light = 6;
        public const int Sound = 7;
        public const int SubScene = 8;
        public const int ScenePlugin = 9;
        public const int SessionPlugin = 10;

        public const int AP =11;
        public const int Force = 12;
        public const int Trigger = 13;
        public const int Refelctive = 14;
        public const int ClothGrab = 15;
        public const int Empty = 16;
        public const int ImagePanel = 17;
        public const int Sign = 18;
        public const int WebPanel = 19;

        private const int maxEnumVal = 19;

        public static string Label (int value)
        {
            switch (value)
            {
                case All:
                    return ("All");
                case General:
                    return ("General");
                case PersonFemale:
                    return ("Person (Female)");
                case PersonMale:
                    return ("Person (Male)");
                case PersonAny:
                    return ("Person (Any)");
                case CUA:
                    return ("Custom Unity Asset");
                case Light:
                    return ("Light");
                case Sound:
                    return ("Sound");
                case SubScene:
                    return ("SubScene");
                case ScenePlugin:
                    return ("Scene Plugin");
                case SessionPlugin:
                    return ("Session Plugin");
                case AP:
                    return ("Animation Pattern");
                case Force:
                    return ("Force");
                case Trigger:
                    return ("Trigger");
                case Refelctive:
                    return ("Reflective");
                case ClothGrab:
                    return ("Cloth Grab");
                case Empty:
                    return ("Empty");
                case ImagePanel:
                    return ("Image Panel");
                case Sign:
                    return ("Sign");
                case WebPanel:
                    return ("Web Panel");
            }
            return ("");
        }
        public static int Enumerate(string value)
        {
            for (int i = 0; i <= maxEnumVal; i++)
            {
                string label = Label(i);
                if (label == value) return i;
            }
            return (-1);
        }

        public static List<string> GetAllTypes(bool typeEnum=false)
        {
            List<string> list = new List<string>();
            for (int i = 1; i <= maxEnumVal; i++)
            {
                string label = Label(i);
                if (label != "")
                {
                    if (!typeEnum) list.Add(label);
                    else list.Add(i.ToString());
                }

            }
            return (list);
        }

        public static List<int> GetAllTypeEnums()
        {
            List<int> list = new List<int>();
            for (int i = 1; i <= maxEnumVal; i++)
            {
                if (Label(i) != "")list.Add(i);
            }

            return (list);
        }

        public static bool MatchType(Atom atom, int type)
        {
            switch (type)
            {
                case General:
                    return (true);
                case PersonFemale:
                    if (atom.type == "Person" && !Utils.IsMale(atom)) return (true);
                    return (false);
                case PersonMale:
                    if (atom.type == "Person" && Utils.IsMale(atom)) return (true);
                    return (false);
                case PersonAny:
                    if (atom.type == "Person") return (true);
                    return (false);
                case CUA:
                    if (atom.type == "CustomUnityAsset") return (true);
                    return (false);
                case Light:
                    if (atom.category == "Light") return (true);
                    return (false);
                case Sound:
                    if (atom.category == "Sound" || atom.type == "Person") return (true);
                    return (false);
                case SubScene:
                    if (atom.type == "SubScene") return (true);
                    return (false);
                case ScenePlugin:
                    if (atom.type == "CoreControl" && atom.type != "SessionPluginManager") return (true);
                    return (false);
                case SessionPlugin:
                    if (atom.type == "CoreControl" && atom.type == "SessionPluginManager") return (true);
                    return (false);
                case AP:
                    if (atom.type == "AnimationPattern") return (true);
                    return (false);
                case Force:
                    if (atom.category == "Force") return (true);
                    return (false);
                case Trigger:
                    if (atom.category == "Triggers") return (true);
                    return (false);
                case Refelctive:
                    if (atom.category == "Reflective") return (true);
                    return (false);
                case ClothGrab:
                    if (atom.type == "ClothGrapSphere") return (true);
                    return (false);
                case Empty:
                    if (atom.type == "Empty") return (true);
                    return (false);
                case ImagePanel:
                    if (atom.type == "ImagePanel"|| atom.type == "ImagePanelEmissive"|| atom.type == "ImagePanelTransparent"|| atom.type == "ImagePanelTransparentEmissive") return (true);
                    return (false);
                case Sign:
                    if (atom.type == "SimpleSign" || atom.type == "UIText" | atom.type == "VaMLogo" || atom.type == "VaMSign") return (true);
                    return (false);
                case WebPanel:
                    if (atom.type == "WebBrowser" || atom.type == "WebPanel" || atom.type == "WebPanelEmissive") return (true);
                    return (false);
            }

            return (false);
        }

        public static List<string> GetMatchingTypes(List<Atom> atoms, bool includeSessionType=true,bool typeEnum=false, bool subGeneralForAll = false)
        {
            List<string> typesList = new List<string>();

            for (int i= 1;i<= maxEnumVal; i++)
            {
                if (i == SessionPlugin)
                {
                    if (!typeEnum) typesList.Add(Label(i));
                    else typesList.Add(i.ToString());
                }
                else if (i == General)
                {
                    if (!typeEnum && subGeneralForAll) typesList.Add("Any");
                    else if (!typeEnum) typesList.Add(Label(i));
                    else typesList.Add(i.ToString());
                }
                else
                {
                    foreach (Atom atom in atoms)
                    {
                        if (MatchType(atom, i))
                        {
                            if (!typeEnum) typesList.Add(Label(i));
                            else typesList.Add(i.ToString());
                            break;
                        }
                    }
                }
            }

            return (typesList);
        }
        public static List<string> GetMatchingAtomNames(List<Atom> atoms, int type, bool excludeSpecialAtoms=true)
        {
            List<string> atomsList = new List<string>();

            if (atoms.Count == 0 || type == SessionPlugin || type == ScenePlugin) atomsList.Add("No matching Atoms found");
            else
            {
                foreach (Atom atom in atoms)
                {
                    if (excludeSpecialAtoms && (atom.name == "[CameraRig]" || atom.name == "CoreControl")) ;
                    else if (MatchType(atom, type)) atomsList.Add(atom.name);
                }
            }
            atomsList.Sort();
            return (atomsList);
        }
    }

}
