using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class PluginAssist : MVRScript
    {
        public PluginManifest pluginManifest;
        public ScenePluginManifest scenePluginManifest;
        public TagsManifest tagsManifest;
        public Settings settings;
        private TabSelectionUI tabSelectionUI;

        SuperController.OnAtomUIDRename atomRenameHandle;
        SuperController.OnAtomRemoved atomRemoveHandle;
        SuperController.OnAtomAdded atomAddedHandle;
        SuperController.OnSceneLoaded sceneLoadedHandle;
        public override void Init()
        {
            try
            {
                if (containingAtom.name == "CoreControl" && containingAtom.type == "SessionPluginManager")
                {
                    settings = new Settings();
                    Utils.Init(settings);
                    Utils.CreatePluginDataFolder();
                    settings.LoadSettings();

                    tagsManifest = new TagsManifest();
                    pluginManifest = new PluginManifest(tagsManifest, settings);
                    scenePluginManifest = new ScenePluginManifest(this);

                    atomRenameHandle = new SuperController.OnAtomUIDRename(this.OnAtomNameUpdate);
                    SuperController.singleton.onAtomUIDRenameHandlers += atomRenameHandle;

                    atomRemoveHandle = new SuperController.OnAtomRemoved(this.OnAtomRemoved);
                    SuperController.singleton.onAtomRemovedHandlers += atomRemoveHandle;

                    sceneLoadedHandle = new SuperController.OnSceneLoaded(OnSceneLoaded);
                    SuperController.singleton.onSceneLoadedHandlers += sceneLoadedHandle;

                    atomAddedHandle = new SuperController.OnAtomAdded(OnAtomAdded);
                    SuperController.singleton.onAtomAddedHandlers += atomAddedHandle;

                    // Initiate integration with Acidbubbles KeyBindings
                    SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);

                    tabSelectionUI = new TabSelectionUI(this);
                }
                else
                {
                    JSONStorableString text = new JSONStorableString("text", "Plugin Assist must be loaded as a Session Plugin");
                    CreateTextField(text);
                }

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(PluginAssist)}.{nameof(Init)}: {e}");
            }
        }

        public void OnBindingsListRequested(List<object> bindings)
        {
            int pluginSlot = int.Parse(this.name.Substring(7, this.name.IndexOf('_') - 7));

            bindings.Add(new JSONStorableAction("Open Load Plugins UI", () =>
            {
                Utils.SelectAndOpenUI(containingAtom, pluginSlot);
                tabSelectionUI.TopTabMenuSelected(TopMenuTabSelection.loadPlugins);
            }
            ));
            bindings.Add(new JSONStorableAction("Open Find Plugins UI", () =>
            {
                Utils.SelectAndOpenUI(containingAtom, pluginSlot);
                tabSelectionUI.TopTabMenuSelected(TopMenuTabSelection.findPlugins);

            }
            ));
        }

        private void OnSceneLoaded()
        {
            scenePluginManifest.RefreshManifest();
        }
        private void OnAtomAdded(Atom atom)
        {
            MVRPluginManager manager = atom.GetStorableByID("PluginManager") as MVRPluginManager;
            if (!SuperController.singleton.isLoading) scenePluginManifest.RefreshManifestForAtom(atom, manager);
        }

        private void OnAtomNameUpdate(string oldName, string newName)
        {
            tabSelectionUI.AtomNameUpdated(oldName, newName);
        }

        private void OnAtomRemoved(Atom atom)
        {
            scenePluginManifest.RemoveAtomFromManifest(atom);
            tabSelectionUI.AtomRemoved(atom);
        }

        public void OnEnable()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(PluginAssist)}.{nameof(OnEnable)}: {e}");
            }
        }

        public void OnDisable()
        {
            try
            {

            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(PluginAssist)}.{nameof(OnDisable)}: {e}");
            }
        }

        public void OnDestroy()
        {
            try
            {
                SuperController.singleton.onAtomUIDRenameHandlers -= atomRenameHandle;
                SuperController.singleton.onAtomRemovedHandlers -= atomRemoveHandle;
                SuperController.singleton.onAtomAddedHandlers -= atomAddedHandle;
                SuperController.singleton.onSceneLoadedHandlers -= sceneLoadedHandle;

                //Deregisters with Acidbubbles KeyBindings plugin
                SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(PluginAssist)}.{nameof(OnDestroy)}: {e}");
            }
        }


        void Start()
        {

        }
    }

}
