﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    public class Settings
    {
        public JSONStorableBool preferredFilterItemsTagEnabled;
        public JSONStorableBool concealedFilterItemsTagEnabled;

        public JSONStorableBool hideHelpText;
        public JSONStorableBool showDebugLogging;

        public JSONStorableBool verboseDebugLogging;

        public Settings()
        {
            preferredFilterItemsTagEnabled = new JSONStorableBool("Display preferred Plugins & Creators with asterisk", true, SettingsBoolChange);
            concealedFilterItemsTagEnabled = new JSONStorableBool("Display concealed Plugins & Creators in brackets", true, SettingsBoolChange);
            hideHelpText = new JSONStorableBool("Hide Help information boxes", false, SettingsBoolChange);
            showDebugLogging = new JSONStorableBool("Enable Debug Logging", false, SettingsBoolChange);
            verboseDebugLogging = new JSONStorableBool("Verbose Debug Logging", false, SettingsBoolChange);
        }

        private void SettingsBoolChange(bool val)
        {
            SaveSettings();
        }

        private void SaveSettings()
        {
            JSONClass settingsJSON = new JSONClass();
            settingsJSON["PAversion"] = Const.PAversion;
            settingsJSON["PAversionFloat"].AsFloat = Const.PAversionFloat;
            settingsJSON["SettingsDataStoreFormat"].AsInt = Const.SettingsDataStoreFormat;
            settingsJSON["preferredFilterItemsTagEnabled"].AsBool = preferredFilterItemsTagEnabled.val;
            settingsJSON["concealedFilterItemsTagEnabled"].AsBool = concealedFilterItemsTagEnabled.val;
            settingsJSON["hideHelpText"].AsBool = hideHelpText.val;
            settingsJSON["showDebugLogging"].AsBool = showDebugLogging.val;
            settingsJSON["verboseDebugLogging"].AsBool = verboseDebugLogging.val;

            SuperController.singleton.SaveJSON(settingsJSON, Const.settingsFolder + "\\" + Const.settingsFileName);
        }

        public void LoadSettings()
        {
            if (FileManagerSecure.FileExists(Const.settingsFolder + "\\" + Const.settingsFileName))
            {
                JSONClass settingsJSON = (JSONClass) SuperController.singleton.LoadJSON(Const.settingsFolder + "\\" + Const.settingsFileName);
                if (settingsJSON["SettingsDataStoreFormat"] != null && settingsJSON["SettingsDataStoreFormat"].AsInt > Const.SettingsDataStoreFormat) Utils.LogDebug("<LoadSettings> Settings Store Format is a higher version than supported.");
                if (settingsJSON["preferredFilterItemsTagEnabled"] != null) preferredFilterItemsTagEnabled.val = settingsJSON["preferredFilterItemsTagEnabled"].AsBool;
                if (settingsJSON["concealedFilterItemsTagEnabled"] != null) concealedFilterItemsTagEnabled.val = settingsJSON["concealedFilterItemsTagEnabled"].AsBool;
                if (settingsJSON["hideHelpText"] != null) hideHelpText.val = settingsJSON["hideHelpText"].AsBool;
                if (settingsJSON["showDebugLogging"] != null) showDebugLogging.val = settingsJSON["showDebugLogging"].AsBool;
                if (settingsJSON["verboseDebugLogging"] != null) verboseDebugLogging.val = settingsJSON["verboseDebugLogging"].AsBool;
            }
           
        }
    }

}
