using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;

namespace JayJayWon
{
    static class Const
    {
        public const string VARPluginMetaDataFolder = "Saves\\PluginData\\JayJayWon\\PluginAssist\\VARPluginMetaData";
        public const string VARDefaultPluginMetaDataFolder = "JayJayWon.PluginAssist.latest:\\Saves\\PluginData\\JayJayWon\\PluginAssist\\VARPluginMetaDataDefault";
        public const string LocalPluginMetaDataFolder = "Saves\\PluginData\\JayJayWon\\PluginAssist\\LocalPluginMetaData";
        public const string CreatorMetaDataFolder = "Saves\\PluginData\\JayJayWon\\PluginAssist\\CreatorMetaData";
        public const string CreatorDefaultMetaDataFolder = "JayJayWon.PluginAssist.latest:\\Saves\\PluginData\\JayJayWon\\PluginAssist\\CreatorMetaDataDefault";
        public const string PAversion = "2.3";
        public const float PAversionFloat = 2.03f;
        public const int MetaDataStoreFormat = 1;
        public const bool EnablePatronContent = true;
        public const string CreatorMetaDataFileExt = ".creatorManifest";
        public const string PluginMetaDataFileExt = ".pluginManifest";
        public const string PluginFolderVariantMetaDataFileExt = ".pvmf";

        public const string settingsFileName = "current.pasettings";
        public const string settingsFolder = "Saves\\PluginData\\JayJayWon\\PluginAssist";
        public const int SettingsDataStoreFormat = 1;

        public const string tagsFileName = "tags.paManifest";
        public const string tagsFolder = "Saves\\PluginData\\JayJayWon\\PluginAssist";
        public const int TagsDataStoreFormat = 1;
    }

    static class HelpText
    {
        public const string filterPluginsHelpText = "The left panel is used to select a Plugin. Plugins can be filtered using:\n\nCreator: author of the VAR package containing the plugin. Select 'Local' to filter for non VAR packaged plugins under the Custom\\Scripts folder\n\nPlugin Type: type of Atom a plugin should be loaded into. Plugins can be assigned a Type under Categorisation\n\nPlugin Tags: user definable labels (JayJayWon Patrons only). Tags can be assigned to Plugins under Categorisation\n\nDepending on the selected plugin, different VAR versions (or variants of the plugin in different folders) can also be selected.";
        public const string loadPluginsHelpText = "The right panel is used to Load the currently selected Plugin (displayed in the top text box) into a target atom.\n\nTarget atoms can be filtered by available atom types from the current scene.\n\nBelow the Load Plugin button is a toggle option to open the plugin UI after loading into the target atom (activate before loading the plugin).";
        public const string findPluginsLeftHelpText = "The left panel is used to select a Plugin from those loaded in the current Scene (or Session). If the selected plugin has multiple loaded instances, then a specific atom and plugin slot (within that atom) can be selected.";
        public const string findPluginsRightHelpText = "The right panel can be used to open the Plugin UI for the plugin selected in the left panel.";

        public const string catPluginsHelpText = "The right panel can be used to categorise the selected plugin. Plugins can be categorised as follows:\n\nPlugin Type: Select the atom type that the plugin is designed to load into. If the plugin supports multiple atom types then use the General type.\n\nPreferred Plugin (Variant): Sets the plugin (or folder variant of the the plugin) as preferred, moving it to the top of the selection list\n\nConceal Plugin (Variant): Sets the plugin (or folder variant of the the plugin) as concealled, hiding it in the plugin selection list\n\nPluginTags: Assign multiple plugin tags to the plugin. Tags are user definable in the Modify Plug Tags screen.";
        public const string catCreatorsHelpText = "The right panel can be used to categorise the Creator of the selected plugin. Creators can be categorised as follows:\n\nPreferred Creator: Sets the Creator as preferred, moving the Creator to the top of selection lists\n\nConceal Creator: Sets the Creator as concealled, hiding the Creator and their plugins from the selection lists\n\n";
        public const string catPluginTagsHelpText = "User definable Tags can be created, edited or deleted. Editing the name of a Tag will modify the tag name in all plugins that have been categorised with that tag.\n\nTAG DELETION WILL INSTANTLY REMOVE THE TAG FROM ALL PLUGINS AND CAN NOT BE UNDONE";
        public const string setPluginFiltersHelpText = "Settings that relate to the plugin filters-\n\nDisplay concealed Plugins & Creators in brackets: when enabled, alters how concealed items are shown in the Categorisation screens for Plugins & Creators. Concealed items will be highlighted by being encapsulated in brackets e.g. (JayJayWon)\n\nDisplay preferred Plugins & Creators with asterix: when enabled, alters how preferred items are shown in all screens that contain a plugin filter. Preferred items will be highlighted by being encapsulated in asterix symbols e.g. * JayJayWon *";
        public const string setGeneralLeftHelpText = "The Refresh Manifest buttons can be used to rescan all packaged VARs (or the Local \\Custom\\Scripts folder) to catalogue available Plugins. These scans are automatically performed every time PluginAssist starts, but if new plugins are introduced a manual rescan is required for them to be listed in the Plugin Assist UI";
        public const string setGeneralRightHelpText = "Two options are available in the right panel-\n\nHide help information boxes: When enabled, these help boxes will be hidden in all screens.\n\nEnable debug logging: When enabled, provides some more verbose logging if problems are detected";
        public const string tagsNonePatronText = "Plugin Tags are a feature available only for JayJayWon Patrons. Plugin Tags are user defined, customizable labels. Each plugin can be assigned multiple tags. Plugin searching can then be filtered by a specific tag.\n\nSupport JayJayWon at https://www.patreon.com/JayJayWon";

    }

        static class Utils
    {
        static private Settings settings;
        static public void Init(Settings _settings)
        {
            settings = _settings;
        }
        static public bool CreatePluginDataFolder()
        {
            bool result = false;
            if (FileManagerSecure.DirectoryExists("Saves\\PluginData"))
            {
                if (!FileManagerSecure.DirectoryExists(Const.VARPluginMetaDataFolder))
                {
                    FileManagerSecure.CreateDirectory(Const.VARPluginMetaDataFolder);
                    string varDefaultPluginMetaDataFolder= SuperController.singleton.NormalizeLoadPath(Const.VARDefaultPluginMetaDataFolder);
                    if (FileManagerSecure.DirectoryExists(varDefaultPluginMetaDataFolder))
                    {
                        foreach (string filePath in FileManagerSecure.GetFiles(varDefaultPluginMetaDataFolder)) {
                            string fileName = FileManagerSecure.GetFileName(filePath);
                            FileManagerSecure.CopyFile(filePath,Const.VARPluginMetaDataFolder+"\\"+fileName);
                        }
                    }
                }
                if (!FileManagerSecure.DirectoryExists(Const.LocalPluginMetaDataFolder)) FileManagerSecure.CreateDirectory(Const.LocalPluginMetaDataFolder);
                if (!FileManagerSecure.DirectoryExists(Const.CreatorMetaDataFolder))
                {
                    FileManagerSecure.CreateDirectory(Const.CreatorMetaDataFolder);
                    string varDefaultCreatorMetaDataFolder = SuperController.singleton.NormalizeLoadPath(Const.CreatorDefaultMetaDataFolder);
                    if (FileManagerSecure.DirectoryExists(varDefaultCreatorMetaDataFolder))
                    {
                        foreach (string filePath in FileManagerSecure.GetFiles(varDefaultCreatorMetaDataFolder))
                        {
                            string fileName = FileManagerSecure.GetFileName(filePath);
                            FileManagerSecure.CopyFile(filePath, Const.CreatorMetaDataFolder + "\\" + fileName);
                        }
                    }
                }
                if (FileManagerSecure.DirectoryExists(Const.VARPluginMetaDataFolder)) result = true;
            }
            else SuperController.LogMessage("PluginAssist: this function requires the folder Saves\\PluginData to exist.");
            return (result);
        }

        static public HashSet<string> GetFilesAtPathRecursive(string path, string pattern, bool suppressVAR = false)
        {
            string normalizedPath = FileManagerSecure.NormalizePath(path);
            HashSet<string> combined = new HashSet<string>();
            string[] files = FileManagerSecure.GetFiles(normalizedPath, pattern);
            string[] directories = FileManagerSecure.GetDirectories(normalizedPath);

            files.ToList().ForEach(file =>
            {
                string normalizedFilePath = FileManagerSecure.NormalizePath(file);
               
                if (suppressVAR) combined.Add(normalizedFilePath.Substring(normalizedFilePath.IndexOf(':') + 2));
                else combined.Add(normalizedFilePath);
            });

            directories.ToList().ForEach(directory =>
            {
                string normalizedSubDirectoryPath = FileManagerSecure.NormalizePath(directory);
 //               SuperController.LogMessage("PATH: " + path + " DIRECTORY: " + directory + " NORMAL: "+ normalizedSubDirectoryPath);
                string folder = normalizedSubDirectoryPath.Remove(0, normalizedPath.Length + 1);

                if (!folder.Contains(".var:")) combined.UnionWith(GetFilesAtPathRecursive(normalizedSubDirectoryPath, pattern, suppressVAR));

            });

            return combined;
        }
        static public void LogDebug(string message, bool verboseOnly = false)
        {
            if (settings.showDebugLogging.val && (!verboseOnly || settings.verboseDebugLogging.val)) SuperController.LogMessage("PluginAssist: " + message);
        }

        static public bool IsMale(Atom atom)
        {
            bool isMale = false;
            // If the peson atom is not "On", then we cant determine their gender it seems as GetComponentInChildren<DAZCharacter> just returns null
            if (atom.on) isMale = atom.GetComponentInChildren<DAZCharacter>().name.StartsWith("male") ||
                    atom.GetComponentInChildren<DAZCharacter>().name.StartsWith("jarlee") ||
                    atom.GetComponentInChildren<DAZCharacter>().name.StartsWith("jarjulian") ||
                    atom.GetComponentInChildren<DAZCharacter>().name.StartsWith("Futa");
            return (isMale);
        }

        static public string GetFolderFromPath(string path)
        {
            string folder = FileManagerSecure.GetDirectoryName(path, true);
            if (folder.Contains(":")) return(folder.Substring(folder.IndexOf(':')+2));
            return (folder);
        }

        static public void SortLists(ref List<string> sortByList, ref List<string> secondList)
        {
            var tempSortByList = sortByList;
            if (sortByList.Count > 0 && secondList.Count == sortByList.Count)
            {
                var combinedChoices = secondList.Select((choice, i) => new { choice, sortChoice = tempSortByList[i] });
                var sortedCombinedChoices = combinedChoices.OrderBy(a => a.sortChoice);

                secondList = sortedCombinedChoices.Select(a => a.choice).ToList();
                sortByList = sortedCombinedChoices.Select(a => a.sortChoice).ToList();
            }
            else if (secondList.Count != sortByList.Count) Utils.LogDebug("SortLists: first and second list not same length");
        }

        static public void SelectAndOpenUI(Atom atom, int pluginSlot, string pluginUIName="")
        {
            if (atom.name == "CoreControl" || atom.type == "SessionPluginManager")
            {
                SuperController.singleton.ShowMainHUDMonitor();
                SuperController.singleton.activeUI = SuperController.ActiveUI.MainMenu;
            }
            else
            {
                SuperController.singleton.SelectController(atom.mainController, false, false, true);
                SuperController.singleton.ShowMainHUDMonitor();
            }

            SuperController.singleton.StartCoroutine(WaitForUI(atom, pluginSlot, pluginUIName));
        }

        static private IEnumerator WaitForUI(Atom atom, int pluginSlot, string pluginUIName)
        {
            var expiration = Time.unscaledTime + 1f;
            UITabSelector selector = null;
            while (Time.unscaledTime < expiration)
            {
                yield return 0;
                if (atom.name == "CoreControl" || atom.type == "SessionPluginManager") selector = SuperController.singleton.mainMenuTabSelector;
                else selector = atom.gameObject.GetComponentInChildren<UITabSelector>();
                if (selector == null) continue;

                if (atom.type == "SessionPluginManager") SuperController.singleton.mainMenuTabSelector.SetActiveTab("TabSessionPlugins");
                else if (atom.name == "CoreControl") SuperController.singleton.mainMenuTabSelector.SetActiveTab("TabScenePlugins");
                else selector.SetActiveTab("Plugins");

                IEnumerator enumerator1 = selector.transform.GetEnumerator();
                while (enumerator1.MoveNext())
                {
                    UITab component = ((Component)enumerator1.Current).GetComponent<UITab>();

                    if ((UnityEngine.Object)component != (UnityEngine.Object)null)
                    {
                        foreach (var scriptUI in component.GetComponentsInChildren<MVRScriptUI>())
                        {
                            scriptUI.closeButton?.onClick.Invoke();
                        }

                        foreach (var scriptController in component.GetComponentsInChildren<MVRScriptControllerUI>())
                        {
                            if ((pluginUIName =="" && scriptController.label.text.StartsWith("plugin#" + pluginSlot.ToString())) || scriptController.label.text == ("plugin#" + pluginSlot.ToString() + "_" + pluginUIName))
                            {
                                scriptController.openUIButton?.onClick?.Invoke();
                                break;
                            }
                        }

                    }
                }

                yield break;
            }
        }


    }
}
